@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -71,11 +71,10 @@ package RT::Attachments;
 use strict;
 use warnings;
 
+use base 'RT::SearchBuilder';
 
 use RT::Attachment;
 
-use base 'RT::SearchBuilder';
-
 sub Table { 'Attachments'}
 
 
@@ -112,14 +111,12 @@ sub TransactionAlias {
     return $self->{'_sql_transaction_alias'}
         if $self->{'_sql_transaction_alias'};
 
-    my $res = $self->NewAlias('Transactions');
-    $self->Limit(
-        ENTRYAGGREGATOR => 'AND',
-        FIELD           => 'TransactionId',
-        VALUE           => $res . '.id',
-        QUOTEVALUE      => 0,
+    return $self->{'_sql_transaction_alias'} = $self->Join(
+        ALIAS1 => 'main',
+        FIELD1 => 'TransactionId',
+        TABLE2 => 'Transactions',
+        FIELD2 => 'id',
     );
-    return $self->{'_sql_transaction_alias'} = $res;
 }
 
 =head2 ContentType (VALUE => 'text/plain', ENTRYAGGREGATOR => 'OR', OPERATOR => '=' ) 
@@ -133,9 +130,9 @@ sub ContentType  {
     my $self = shift;
     my %args = (
         VALUE           => 'text/plain',
-	    OPERATOR        => '=',
-	    ENTRYAGGREGATOR => 'OR',
-	    @_
+        OPERATOR        => '=',
+        ENTRYAGGREGATOR => 'OR',
+        @_
     );
 
     return $self->Limit ( %args, FIELD => 'ContentType' );
@@ -185,6 +182,35 @@ sub LimitNotEmpty {
     return;
 }
 
+=head2 LimitHasFilename
+
+Limit result set to attachments with not empty filename.
+
+=cut
+
+sub LimitHasFilename {
+    my $self = shift;
+
+    $self->Limit(
+        ENTRYAGGREGATOR => 'AND',
+        FIELD           => 'Filename',
+        OPERATOR        => 'IS NOT',
+        VALUE           => 'NULL',
+        QUOTEVALUE      => 0,
+    );
+
+    if ( RT->Config->Get('DatabaseType') ne 'Oracle' ) {
+        $self->Limit(
+            ENTRYAGGREGATOR => 'AND',
+            FIELD           => 'Filename',
+            OPERATOR        => '!=',
+            VALUE           => '',
+        );
+    }
+
+    return;
+}
+
 =head2 LimitByTicket $ticket_id
 
 Limit result set to attachments of a ticket.
@@ -203,13 +229,11 @@ sub LimitByTicket {
         VALUE           => 'RT::Ticket',
     );
 
-    my $tickets = $self->NewAlias('Tickets');
-    $self->Limit(
-        ENTRYAGGREGATOR => 'AND',
-        ALIAS           => $tickets,
-        FIELD           => 'id',
-        VALUE           => $transactions . '.ObjectId',
-        QUOTEVALUE      => 0,
+    my $tickets = $self->Join(
+        ALIAS1 => $transactions,
+        FIELD1 => 'ObjectId',
+        TABLE2 => 'Tickets',
+        FIELD2 => 'id',
     );
     $self->Limit(
         ENTRYAGGREGATOR => 'AND',
@@ -228,18 +252,6 @@ sub AddRecord {
     return $self->SUPER::AddRecord( $record );
 }
 
-
-=head2 NewItem
-
-Returns an empty new RT::Attachment item
-
-=cut
-
-sub NewItem {
-    my $self = shift;
-    return(RT::Attachment->new($self->CurrentUser));
-}
-
 RT::Base->_ImportOverlays();
 
 1;
